@@ -1,11 +1,12 @@
 import numpy as np
-import scipy.sparse as sparse
 
 from skimage.metrics import (adapted_rand_error,
                              variation_of_information,
                              contingency_table)
 
-from skimage._shared.testing import assert_equal, assert_almost_equal, assert_array_equal
+from skimage._shared.testing import (assert_equal,
+                                     assert_almost_equal,
+                                     assert_array_equal)
 
 
 def test_contingency_table():
@@ -29,6 +30,17 @@ def test_vi():
     assert_equal(np.sum(variation_of_information(im_true, im_test)), 1)
 
 
+def test_vi_ignore_labels():
+    im1 = np.array([[1, 0],
+                    [2, 3]], dtype='uint8')
+    im2 = np.array([[1, 1],
+                    [1, 0]], dtype='uint8')
+
+    false_splits, false_merges = variation_of_information(im1, im2,
+                                                          ignore_labels=[0])
+    assert (false_splits, false_merges) == (0, 2 / 3)
+
+
 def test_are():
     im_true = np.array([[2, 1], [1, 2]])
     im_test = np.array([[1, 2], [3, 1]])
