@@ -1,13 +1,12 @@
 import numpy as np
-from skimage.segmentation import (morphological_chan_vese,
-                                  morphological_geodesic_active_contour,
-                                  inverse_gaussian_gradient,
-                                  circle_level_set,
-                                  disk_level_set)
+import pytest
+from numpy.testing import assert_array_equal
 
-from skimage._shared import testing
-from skimage._shared.testing import assert_array_equal
 from skimage._shared._warnings import expected_warnings
+from skimage.segmentation import (disk_level_set,
+                                  inverse_gaussian_gradient,
+                                  morphological_chan_vese,
+                                  morphological_geodesic_active_contour)
 
 
 def gaussian_blob():
@@ -20,10 +19,10 @@ def test_morphsnakes_incorrect_image_shape():
     img = np.zeros((10, 10, 3))
     ls = np.zeros((10, 9))
 
-    with testing.raises(ValueError):
-        morphological_chan_vese(img, iterations=1, init_level_set=ls)
-    with testing.raises(ValueError):
-        morphological_geodesic_active_contour(img, iterations=1,
+    with pytest.raises(ValueError):
+        morphological_chan_vese(img, num_iter=1, init_level_set=ls)
+    with pytest.raises(ValueError):
+        morphological_geodesic_active_contour(img, num_iter=1,
                                               init_level_set=ls)
 
 
@@ -31,10 +30,10 @@ def test_morphsnakes_incorrect_ndim():
     img = np.zeros((4, 4, 4, 4))
     ls = np.zeros((4, 4, 4, 4))
 
-    with testing.raises(ValueError):
-        morphological_chan_vese(img, iterations=1, init_level_set=ls)
-    with testing.raises(ValueError):
-        morphological_geodesic_active_contour(img, iterations=1,
+    with pytest.raises(ValueError):
+        morphological_chan_vese(img, num_iter=1, init_level_set=ls)
+    with pytest.raises(ValueError):
+        morphological_geodesic_active_contour(img, num_iter=1,
                                               init_level_set=ls)
 
 
@@ -45,14 +44,14 @@ def test_morphsnakes_black():
     ref_zeros = np.zeros(img.shape, dtype=np.int8)
     ref_ones = np.ones(img.shape, dtype=np.int8)
 
-    acwe_ls = morphological_chan_vese(img, iterations=6, init_level_set=ls)
+    acwe_ls = morphological_chan_vese(img, num_iter=6, init_level_set=ls)
     assert_array_equal(acwe_ls, ref_zeros)
 
-    gac_ls = morphological_geodesic_active_contour(img, iterations=6,
+    gac_ls = morphological_geodesic_active_contour(img, num_iter=6,
                                                    init_level_set=ls)
     assert_array_equal(gac_ls, ref_zeros)
 
-    gac_ls2 = morphological_geodesic_active_contour(img, iterations=6,
+    gac_ls2 = morphological_geodesic_active_contour(img, num_iter=6,
                                                     init_level_set=ls,
                                                     balloon=1, threshold=-1,
                                                     smoothing=0)
@@ -61,13 +60,29 @@ def test_morphsnakes_black():
     assert acwe_ls.dtype == gac_ls.dtype == gac_ls2.dtype == np.int8
 
 
+def test_morphsnakes_iterations_kwarg_deprecation():
+    img = np.zeros((11, 11))
+    ls = disk_level_set(img.shape, center=(5, 5), radius=3)
+
+    ref_zeros = np.zeros(img.shape, dtype=np.int8)
+
+    with expected_warnings(["`iterations` is a deprecated argument"]):
+        acwe_ls = morphological_chan_vese(img, iterations=6, init_level_set=ls)
+    assert_array_equal(acwe_ls, ref_zeros)
+
+    with expected_warnings(["`iterations` is a deprecated argument"]):
+        gac_ls = morphological_geodesic_active_contour(img, iterations=6,
+                                                       init_level_set=ls)
+    assert_array_equal(gac_ls, ref_zeros)
+
+
 def test_morphsnakes_simple_shape_chan_vese():
     img = gaussian_blob()
     ls1 = disk_level_set(img.shape, center=(5, 5), radius=3)
     ls2 = disk_level_set(img.shape, center=(5, 5), radius=6)
 
-    acwe_ls1 = morphological_chan_vese(img, iterations=10, init_level_set=ls1)
-    acwe_ls2 = morphological_chan_vese(img, iterations=10, init_level_set=ls2)
+    acwe_ls1 = morphological_chan_vese(img, num_iter=10, init_level_set=ls1)
+    acwe_ls2 = morphological_chan_vese(img, num_iter=10, init_level_set=ls2)
 
     assert_array_equal(acwe_ls1, acwe_ls2)
 
@@ -75,7 +90,7 @@ def test_morphsnakes_simple_shape_chan_vese():
 
 
 def test_morphsnakes_simple_shape_geodesic_active_contour():
-    img = np.float_(disk_level_set((11, 11), center=(5, 5), radius=3.5))
+    img = (disk_level_set((11, 11), center=(5, 5), radius=3.5)).astype(float)
     gimg = inverse_gaussian_gradient(img, alpha=10.0, sigma=1.0)
     ls = disk_level_set(img.shape, center=(5, 5), radius=6)
 
@@ -92,19 +107,13 @@ def test_morphsnakes_simple_shape_geodesic_active_contour():
                     [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
                    dtype=np.int8)
 
-    gac_ls = morphological_geodesic_active_contour(gimg, iterations=10,
+    gac_ls = morphological_geodesic_active_contour(gimg, num_iter=10,
                                                    init_level_set=ls,
                                                    balloon=-1)
     assert_array_equal(gac_ls, ref)
     assert gac_ls.dtype == np.int8
 
 
-def test_deprecated_circle_level_set():
-    img = gaussian_blob()
-    with expected_warnings(['circle_level_set is deprecated']):
-        ls1 = circle_level_set(img.shape, (5, 5), 3)
-
-
 def test_init_level_sets():
     image = np.zeros((6, 6))
     checkerboard_ls = morphological_chan_vese(image, 0, 'checkerboard')
