@@ -1,13 +1,18 @@
 import numpy as np
+from numpy.lib import NumpyVersion
+import scipy
 from scipy import ndimage as ndi
 
 from ._geometric import (SimilarityTransform, AffineTransform,
-                         ProjectiveTransform, _to_ndimage_mode)
+                         ProjectiveTransform)
 from ._warps_cy import _warp_fast
 from ..measure import block_reduce
 
 from .._shared.utils import (get_bound_method_class, safe_as_int, warn,
-                             convert_to_float, _validate_interpolation_order)
+                             convert_to_float, _to_ndimage_mode,
+                             _validate_interpolation_order,
+                             channel_as_last_axis,
+                             deprecate_multichannel_kwarg)
 
 HOMOGRAPHY_TRANSFORMS = (
     SimilarityTransform,
@@ -16,20 +21,68 @@ HOMOGRAPHY_TRANSFORMS = (
 )
 
 
+def _preprocess_resize_output_shape(image, output_shape):
+    """Validate resize output shape according to input image.
+
+    Parameters
+    ----------
+    image: ndarray
+        Image to be resized.
+    output_shape: iterable
+        Size of the generated output image `(rows, cols[, ...][, dim])`. If
+        `dim` is not provided, the number of channels is preserved.
+
+    Returns
+    -------
+    image: ndarray
+        The input image, but with additional singleton dimensions appended in
+        the case where ``len(output_shape) > input.ndim``.
+    output_shape: tuple
+        The output image converted to tuple.
+
+    Raises
+    ------
+    ValueError:
+        If output_shape length is smaller than the image number of
+        dimensions
+
+    Notes
+    -----
+    The input image is reshaped if its number of dimensions is not
+    equal to output_shape_length.
+
+    """
+    output_shape = tuple(output_shape)
+    output_ndim = len(output_shape)
+    input_shape = image.shape
+    if output_ndim > image.ndim:
+        # append dimensions to input_shape
+        input_shape += (1, ) * (output_ndim - image.ndim)
+        image = np.reshape(image, input_shape)
+    elif output_ndim == image.ndim - 1:
+        # multichannel case: append shape of last axis
+        output_shape = output_shape + (image.shape[-1], )
+    elif output_ndim < image.ndim:
+        raise ValueError("output_shape length cannot be smaller than the "
+                         "image number of dimensions")
+
+    return image, output_shape
+
+
 def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
            preserve_range=False, anti_aliasing=None, anti_aliasing_sigma=None):
     """Resize image to match a certain size.
 
     Performs interpolation to up-size or down-size N-dimensional images. Note
     that anti-aliasing should be enabled when down-sizing images to avoid
-    aliasing artifacts. For down-sampling with an integer factor also see
+    aliasing artifacts. For downsampling with an integer factor also see
     `skimage.transform.downscale_local_mean`.
 
     Parameters
     ----------
     image : ndarray
         Input image.
-    output_shape : tuple or ndarray
+    output_shape : iterable
         Size of the generated output image `(rows, cols[, ...][, dim])`. If
         `dim` is not provided, the number of channels is preserved. In case the
         number of input channels does not equal the number of output channels a
@@ -62,13 +115,13 @@ def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
         Also see https://scikit-image.org/docs/dev/user_guide/data_types.html
     anti_aliasing : bool, optional
         Whether to apply a Gaussian filter to smooth the image prior
-        to down-scaling. It is crucial to filter when down-sampling
-        the image to avoid aliasing artifacts. If input image data
-        type is bool, no anti-aliasing is applied.
+        to downsampling. It is crucial to filter when downsampling
+        the image to avoid aliasing artifacts. If not specified, it is set to
+        True when downsampling an image whose data type is not bool.
     anti_aliasing_sigma : {float, tuple of floats}, optional
-        Standard deviation for Gaussian filtering to avoid aliasing artifacts.
+        Standard deviation for Gaussian filtering used when anti-aliasing.
         By default, this value is chosen as (s - 1) / 2 where s is the
-        down-scaling factor, where s > 1. For the up-size case, s < 1, no
+        downsampling factor, where s > 1. For the up-size case, s < 1, no
         anti-aliasing is performed prior to rescaling.
 
     Notes
@@ -88,33 +141,31 @@ def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
     (100, 100)
 
     """
-    output_shape = tuple(output_shape)
-    output_ndim = len(output_shape)
+
+    image, output_shape = _preprocess_resize_output_shape(image, output_shape)
     input_shape = image.shape
-    if output_ndim > image.ndim:
-        # append dimensions to input_shape
-        input_shape = input_shape + (1, ) * (output_ndim - image.ndim)
-        image = np.reshape(image, input_shape)
-    elif output_ndim == image.ndim - 1:
-        # multichannel case: append shape of last axis
-        output_shape = output_shape + (image.shape[-1], )
-    elif output_ndim < image.ndim - 1:
-        raise ValueError("len(output_shape) cannot be smaller than the image "
-                         "dimensions")
+    input_type = image.dtype
+
+    if input_type == np.float16:
+        image = image.astype(np.float32)
 
     if anti_aliasing is None:
-        anti_aliasing = not image.dtype == bool
+        anti_aliasing = (not input_type == bool and
+                         any(x < y for x, y in zip(output_shape, input_shape)))
 
-    if image.dtype == bool and anti_aliasing:
-        warn("Input image dtype is bool. Gaussian convolution is not defined "
-             "with bool data type. Please set anti_aliasing to False or "
-             "explicitely cast input image to another data type. Starting "
-             "from version 0.19 a ValueError will be raised instead of this "
-             "warning.", FutureWarning, stacklevel=2)
+    if input_type == bool and anti_aliasing:
+        raise ValueError("anti_aliasing must be False for boolean images")
 
-    factors = (np.asarray(input_shape, dtype=float) /
-               np.asarray(output_shape, dtype=float))
+    factors = np.divide(input_shape, output_shape)
+    order = _validate_interpolation_order(input_type, order)
+    if order > 0:
+        image = convert_to_float(image, preserve_range)
 
+    # Save input value range for clip
+    img_bounds = np.array([image.min(), image.max()]) if clip else None
+
+    # Translate modes used by np.pad to those used by scipy.ndimage
+    ndi_mode = _to_ndimage_mode(mode)
     if anti_aliasing:
         if anti_aliasing_sigma is None:
             anti_aliasing_sigma = np.maximum(0, (factors - 1) / 2)
@@ -127,29 +178,20 @@ def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
             elif np.any((anti_aliasing_sigma > 0) & (factors <= 1)):
                 warn("Anti-aliasing standard deviation greater than zero but "
                      "not down-sampling along all axes")
-
-        # Translate modes used by np.pad to those used by ndi.gaussian_filter
-        np_pad_to_ndimage = {
-            'constant': 'constant',
-            'edge': 'nearest',
-            'symmetric': 'reflect',
-            'reflect': 'mirror',
-            'wrap': 'wrap'
-        }
-        try:
-            ndi_mode = np_pad_to_ndimage[mode]
-        except KeyError:
-            raise ValueError("Unknown mode, or cannot translate mode. The "
-                             "mode should be one of 'constant', 'edge', "
-                             "'symmetric', 'reflect', or 'wrap'. See the "
-                             "documentation of numpy.pad for more info.")
-
         image = ndi.gaussian_filter(image, anti_aliasing_sigma,
                                     cval=cval, mode=ndi_mode)
 
+    if NumpyVersion(scipy.__version__) >= '1.6.0':
+        # The grid_mode kwarg was introduced in SciPy 1.6.0
+        zoom_factors = [1 / f for f in factors]
+        out = ndi.zoom(image, zoom_factors, order=order, mode=ndi_mode,
+                       cval=cval, grid_mode=True)
+
+    # TODO: Remove the fallback code below once SciPy >= 1.6.0 is required.
+
     # 2-dimensional interpolation
-    if len(output_shape) == 2 or (len(output_shape) == 3 and
-                                  output_shape[2] == input_shape[2]):
+    elif len(output_shape) == 2 or (len(output_shape) == 3 and
+                                    output_shape[2] == input_shape[2]):
         rows = output_shape[0]
         cols = output_shape[1]
         input_rows = input_shape[0]
@@ -173,12 +215,12 @@ def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
         tform.params[0, 1] = 0
         tform.params[1, 0] = 0
 
+        # clip outside of warp to clip w.r.t input values, not filtered values.
         out = warp(image, tform, output_shape=output_shape, order=order,
-                   mode=mode, cval=cval, clip=clip,
+                   mode=mode, cval=cval, clip=False,
                    preserve_range=preserve_range)
 
     else:  # n-dimensional interpolation
-        order = _validate_interpolation_order(image.dtype, order)
 
         coord_arrays = [factors[i] * (np.arange(d) + 0.5) - 0.5
                         for i, d in enumerate(output_shape)]
@@ -187,20 +229,20 @@ def resize(image, output_shape, order=None, mode='reflect', cval=0, clip=True,
                                          sparse=False,
                                          indexing='ij'))
 
-        image = convert_to_float(image, preserve_range)
-
-        ndi_mode = _to_ndimage_mode(mode)
         out = ndi.map_coordinates(image, coord_map, order=order,
                                   mode=ndi_mode, cval=cval)
 
-        _clip_warp_output(image, out, order, mode, cval, clip)
+    _clip_warp_output(img_bounds, out, mode, cval, clip)
 
     return out
 
 
+@channel_as_last_axis()
+@deprecate_multichannel_kwarg(multichannel_position=7)
 def rescale(image, scale, order=None, mode='reflect', cval=0, clip=True,
             preserve_range=False, multichannel=False,
-            anti_aliasing=None, anti_aliasing_sigma=None):
+            anti_aliasing=None, anti_aliasing_sigma=None, *,
+            channel_axis=None):
     """Scale image by a certain factor.
 
     Performs interpolation to up-scale or down-scale N-dimensional images.
@@ -244,7 +286,8 @@ def rescale(image, scale, order=None, mode='reflect', cval=0, clip=True,
         https://scikit-image.org/docs/dev/user_guide/data_types.html
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension.
+        channels or another spatial dimension. This argument is deprecated:
+        specify `channel_axis` instead.
     anti_aliasing : bool, optional
         Whether to apply a Gaussian filter to smooth the image prior
         to down-scaling. It is crucial to filter when down-sampling
@@ -254,6 +297,13 @@ def rescale(image, scale, order=None, mode='reflect', cval=0, clip=True,
         Standard deviation for Gaussian filtering to avoid aliasing artifacts.
         By default, this value is chosen as (s - 1) / 2 where s is the
         down-scaling factor.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
 
     Notes
     -----
@@ -275,6 +325,7 @@ def rescale(image, scale, order=None, mode='reflect', cval=0, clip=True,
 
     """
     scale = np.atleast_1d(scale)
+    multichannel = channel_axis is not None
     if len(scale) > 1:
         if ((not multichannel and len(scale) != image.ndim) or
                 (multichannel and len(scale) != image.ndim - 1)):
@@ -283,7 +334,7 @@ def rescale(image, scale, order=None, mode='reflect', cval=0, clip=True,
         if multichannel:
             scale = np.concatenate((scale, [1]))
     orig_shape = np.asarray(image.shape)
-    output_shape = np.round(scale * orig_shape)
+    output_shape = np.maximum(np.round(scale * orig_shape), 1)
     if multichannel:  # don't scale channel dimension
         output_shape[-1] = orig_shape[-1]
 
@@ -364,6 +415,9 @@ def rotate(image, angle, resize=False, center=None, order=None,
 
     rows, cols = image.shape[0], image.shape[1]
 
+    if image.dtype == np.float16:
+        image = image.astype(np.float32)
+
     # rotation around center
     if center is None:
         center = np.array((cols, rows)) / 2. - 0.5
@@ -635,7 +689,7 @@ def warp_coords(coord_map, shape, dtype=np.float64):
     return coords
 
 
-def _clip_warp_output(input_image, output_image, order, mode, cval, clip):
+def _clip_warp_output(input_image, output_image, mode, cval, clip):
     """Clip output image to range of values of input image.
 
     Note that this function modifies the values of `output_image` in-place
@@ -650,36 +704,45 @@ def _clip_warp_output(input_image, output_image, order, mode, cval, clip):
 
     Other parameters
     ----------------
-    order : int, optional
-        The order of the spline interpolation, default is 1. The order has to
-        be in the range 0-5. See `skimage.transform.warp` for detail.
-    mode : {'constant', 'edge', 'symmetric', 'reflect', 'wrap'}, optional
+    mode : {'constant', 'edge', 'symmetric', 'reflect', 'wrap'}
         Points outside the boundaries of the input are filled according
         to the given mode.  Modes match the behaviour of `numpy.pad`.
-    cval : float, optional
+    cval : float
         Used in conjunction with mode 'constant', the value outside
         the image boundaries.
-    clip : bool, optional
+    clip : bool
         Whether to clip the output to the range of values of the input image.
         This is enabled by default, since higher order interpolation may
         produce values outside the given input range.
 
     """
-    if clip and order != 0:
-        min_val = input_image.min()
-        max_val = input_image.max()
+    if clip:
+        min_val = np.min(input_image)
+        if np.isnan(min_val):
+            # NaNs detected, use NaN-safe min/max
+            min_func = np.nanmin
+            max_func = np.nanmax
+            min_val = min_func(input_image)
+        else:
+            min_func = np.min
+            max_func = np.max
+        max_val = max_func(input_image)
 
-        preserve_cval = (mode == 'constant' and not
-                         (min_val <= cval <= max_val))
+        # Check if cval has been used such that it expands the effective input
+        # range
+        preserve_cval = (mode == 'constant'
+                         and not min_val <= cval <= max_val
+                         and min_func(output_image) <= cval <= max_func(output_image))
 
+        # expand min/max range to account for cval
         if preserve_cval:
-            cval_mask = output_image == cval
+            # cast cval to the same dtype as the input image
+            cval = input_image.dtype.type(cval)
+            min_val = min(min_val, cval)
+            max_val = max(max_val, cval)
 
         np.clip(output_image, min_val, max_val, out=output_image)
 
-        if preserve_cval:
-            output_image[cval_mask] = cval
-
 
 def warp(image, inverse_map, map_args={}, output_shape=None, order=None,
          mode='constant', cval=0., clip=True, preserve_range=False):
@@ -804,7 +867,8 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=None,
     if you want to rescale a 3-D cube, you can do:
 
     >>> cube_shape = np.array([30, 30, 30])
-    >>> cube = np.random.rand(*cube_shape)
+    >>> rng = np.random.default_rng()
+    >>> cube = rng.random(cube_shape)
 
     Setup the coordinate array, that defines the scaling:
 
@@ -829,7 +893,10 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=None,
 
     order = _validate_interpolation_order(image.dtype, order)
 
-    image = convert_to_float(image, preserve_range)
+    if order > 0:
+        image = convert_to_float(image, preserve_range)
+        if image.dtype == np.float16:
+            image = image.astype(np.float32)
 
     input_shape = np.array(image.shape)
 
@@ -851,7 +918,7 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=None,
              "skimage's implementation is fixed, we recommend "
              "to use bi-linear or bi-cubic interpolation instead.")
 
-    if order in (0, 1, 3) and not map_args:
+    if order in (1, 3) and not map_args:
         # use fast Cython version for specific interpolation orders and input
 
         matrix = None
@@ -927,7 +994,7 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=None,
         warped = ndi.map_coordinates(image, coords, prefilter=prefilter,
                                      mode=ndi_mode, order=order, cval=cval)
 
-    _clip_warp_output(image, warped, order, mode, cval, clip)
+    _clip_warp_output(image, warped, mode, cval, clip)
 
     return warped
 
@@ -994,16 +1061,18 @@ def _log_polar_mapping(output_coords, k_angle, k_radius, center):
     return coords
 
 
+@channel_as_last_axis()
+@deprecate_multichannel_kwarg()
 def warp_polar(image, center=None, *, radius=None, output_shape=None,
-               scaling='linear', multichannel=False, **kwargs):
+               scaling='linear', multichannel=False, channel_axis=None,
+               **kwargs):
     """Remap image to polar or log-polar coordinates space.
 
     Parameters
     ----------
     image : ndarray
-        Input image. Only 2-D arrays are accepted by default. If
-        `multichannel=True`, 3-D arrays are accepted and the last axis is
-        interpreted as multiple channels.
+        Input image. Only 2-D arrays are accepted by default. 3-D arrays are
+        accepted if a `channel_axis` is specified.
     center : tuple (row, col), optional
         Point in image that represents the center of the transformation (i.e.,
         the origin in cartesian space). Values can be of type `float`.
@@ -1018,7 +1087,15 @@ def warp_polar(image, center=None, *, radius=None, output_shape=None,
     multichannel : bool, optional
         Whether the image is a 3-D array in which the third axis is to be
         interpreted as multiple channels. If set to `False` (default), only 2-D
-        arrays are accepted.
+        arrays are accepted. This argument is deprecated: specify
+        `channel_axis` instead.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
+
+        .. versionadded:: 0.19
+           ``channel_axis`` was added in 0.19.
     **kwargs : keyword arguments
         Passed to `transform.warp`.
 
@@ -1049,17 +1126,16 @@ def warp_polar(image, center=None, *, radius=None, output_shape=None,
     Perform a log-polar warp on a color image:
 
     >>> image = data.astronaut()
-    >>> warped = warp_polar(image, scaling='log', multichannel=True)
+    >>> warped = warp_polar(image, scaling='log', channel_axis=-1)
     """
+    multichannel = channel_axis is not None
     if image.ndim != 2 and not multichannel:
-        raise ValueError("Input array must be 2 dimensions "
-                         "when `multichannel=False`,"
-                         " got {}".format(image.ndim))
+        raise ValueError(f'Input array must be 2-dimensional when '
+                         f'`channel_axis=None`, got {image.ndim}')
 
     if image.ndim != 3 and multichannel:
-        raise ValueError("Input array must be 3 dimensions "
-                         "when `multichannel=True`,"
-                         " got {}".format(image.ndim))
+        raise ValueError(f'Input array must be 3-dimensional when '
+                         f'`channel_axis` is specified, got {image.ndim}')
 
     if center is None:
         center = (np.array(image.shape)[:2] / 2) - 0.5
@@ -1093,3 +1169,167 @@ def warp_polar(image, center=None, *, radius=None, output_shape=None,
                   output_shape=output_shape, **kwargs)
 
     return warped
+
+
+def _local_mean_weights(old_size, new_size, grid_mode, dtype):
+    """Create a 2D weight matrix for resizing with the local mean.
+
+    Parameters
+    ----------
+    old_size: int
+        Old size.
+    new_size: int
+        New size.
+    grid_mode : bool
+        Whether to use grid data model of pixel/voxel model for
+        average weights computation.
+    dtype: dtype
+        Output array data type.
+
+    Returns
+    -------
+    weights: (new_size, old_size) array
+        Rows sum to 1.
+
+    """
+    if grid_mode:
+        old_breaks = np.linspace(0, old_size, num=old_size + 1, dtype=dtype)
+        new_breaks = np.linspace(0, old_size, num=new_size + 1, dtype=dtype)
+    else:
+        old, new = old_size - 1, new_size - 1
+        old_breaks = np.pad(np.linspace(0.5, old - 0.5, old, dtype=dtype),
+                            1, 'constant', constant_values=(0, old))
+        if new == 0:
+            val = np.inf
+        else:
+            val = 0.5 * old / new
+        new_breaks = np.pad(np.linspace(val, old - val, new, dtype=dtype),
+                            1, 'constant', constant_values=(0, old))
+
+    upper = np.minimum(new_breaks[1:, np.newaxis], old_breaks[np.newaxis, 1:])
+    lower = np.maximum(new_breaks[:-1, np.newaxis],
+                       old_breaks[np.newaxis, :-1])
+
+    weights = np.maximum(upper - lower, 0)
+    weights /= weights.sum(axis=1, keepdims=True)
+
+    return weights
+
+
+def resize_local_mean(image, output_shape, grid_mode=True,
+                      preserve_range=False, *, channel_axis=None):
+    """Resize an array with the local mean / bilinear scaling.
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image. If this is a multichannel image, the axis corresponding
+        to channels should be specified using `channel_axis`
+    output_shape : iterable
+        Size of the generated output image. When `channel_axis` is not None,
+        the `channel_axis` should either be omitted from `output_shape` or the
+        ``output_shape[channel_axis]`` must match
+        ``image.shape[channel_axis]``. If the length of `output_shape` exceeds
+        image.ndim, additional singleton dimensions will be appended to the
+        input ``image`` as needed.
+    grid_mode : bool, optional
+        Defines ``image`` pixels position: if True, pixels are assumed to be at
+        grid intersections, otherwise at cell centers. As a consequence,
+        for example, a 1d signal of length 5 is considered to have length 4
+        when `grid_mode` is False, but length 5 when `grid_mode` is True. See
+        the following visual illustration:
+
+        .. code-block:: text
+
+                | pixel 1 | pixel 2 | pixel 3 | pixel 4 | pixel 5 |
+                     |<-------------------------------------->|
+                                        vs.
+                |<----------------------------------------------->|
+
+        The starting point of the arrow in the diagram above corresponds to
+        coordinate location 0 in each mode.
+    preserve_range : bool, optional
+        Whether to keep the original range of values. Otherwise, the input
+        image is converted according to the conventions of `img_as_float`.
+        Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
+
+    Returns
+    -------
+    resized : ndarray
+        Resized version of the input.
+
+    See Also
+    --------
+    resize, downscale_local_mean
+
+    Notes
+    -----
+    This method is sometimes referred to as "area-based" interpolation or
+    "pixel mixing" interpolation [1]_. When `grid_mode` is True, it is
+    equivalent to using OpenCV's resize with `INTER_AREA` interpolation mode.
+    It is commonly used for image downsizing. If the downsizing factors are
+    integers, then `downscale_local_mean` should be preferred instead.
+
+    References
+    ----------
+    .. [1] http://entropymine.com/imageworsener/pixelmixing/
+
+    Examples
+    --------
+    >>> from skimage import data
+    >>> from skimage.transform import resize_local_mean
+    >>> image = data.camera()
+    >>> resize_local_mean(image, (100, 100)).shape
+    (100, 100)
+
+    """
+    if channel_axis is not None:
+        if channel_axis < -image.ndim or channel_axis >= image.ndim:
+            raise ValueError("invalid channel_axis")
+
+        # move channels to last position
+        image = np.moveaxis(image, channel_axis, -1)
+        nc = image.shape[-1]
+
+        output_ndim = len(output_shape)
+        if output_ndim == image.ndim - 1:
+            # insert channels dimension at the end
+            output_shape = output_shape + (nc,)
+        elif output_ndim == image.ndim:
+            if output_shape[channel_axis] != nc:
+                raise ValueError(
+                    "Cannot reshape along the channel_axis. Use "
+                    "channel_axis=None to reshape along all axes."
+                )
+            # move channels to last position in output_shape
+            channel_axis = channel_axis % image.ndim
+            output_shape = (
+                output_shape[:channel_axis] + output_shape[channel_axis:] +
+                (nc,)
+            )
+        else:
+            raise ValueError(
+                "len(output_shape) must be image.ndim or (image.ndim - 1) "
+                "when a channel_axis is specified."
+            )
+        resized = image
+    else:
+        resized, output_shape = _preprocess_resize_output_shape(image,
+                                                                output_shape)
+    resized = convert_to_float(resized, preserve_range)
+    dtype = resized.dtype
+
+    for axis, (old_size, new_size) in enumerate(zip(image.shape,
+                                                    output_shape)):
+        if old_size == new_size:
+            continue
+        weights = _local_mean_weights(old_size, new_size, grid_mode, dtype)
+        product = np.tensordot(resized, weights, [[axis], [-1]])
+        resized = np.moveaxis(product, -1, axis)
+
+    if channel_axis is not None:
+        # restore channels to original axis
+        resized = np.moveaxis(resized, -1, channel_axis)
+
+    return resized
