@@ -1,16 +1,21 @@
-import numpy as np
 import re
-from skimage.transform._geometric import GeometricTransform
+import textwrap
+
+import numpy as np
+import pytest
+from numpy.testing import (assert_almost_equal, assert_array_almost_equal,
+                           assert_equal)
+
+from skimage.transform._geometric import (_affine_matrix_from_vector,
+                                          _center_and_normalize_points,
+                                          _euler_rotation_matrix,
+                                          GeometricTransform)
 from skimage.transform import (estimate_transform, matrix_transform,
                                EuclideanTransform, SimilarityTransform,
                                AffineTransform, FundamentalMatrixTransform,
                                EssentialMatrixTransform, ProjectiveTransform,
                                PolynomialTransform, PiecewiseAffineTransform)
 
-from skimage._shared import testing
-from skimage._shared.testing import assert_equal, assert_almost_equal
-import textwrap
-
 
 SRC = np.array([
     [-12.3705, -10.5075],
@@ -38,7 +43,7 @@ def test_estimate_transform():
     for tform in ('euclidean', 'similarity', 'affine', 'projective',
                   'polynomial'):
         estimate_transform(tform, SRC[:2, :], DST[:2, :])
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         estimate_transform('foobar', SRC[:2, :], DST[:2, :])
 
 
@@ -62,7 +67,7 @@ def test_euclidean_estimation():
 
     # via estimate method
     tform3 = EuclideanTransform()
-    tform3.estimate(SRC, DST)
+    assert tform3.estimate(SRC, DST)
     assert_almost_equal(tform3.params, tform2.params)
 
 
@@ -109,7 +114,7 @@ def test_similarity_estimation():
 
     # via estimate method
     tform3 = SimilarityTransform()
-    tform3.estimate(SRC, DST)
+    assert tform3.estimate(SRC, DST)
     assert_almost_equal(tform3.params, tform2.params)
 
 
@@ -150,6 +155,19 @@ def test_similarity_init():
     assert_almost_equal(tform.rotation, rotation)
     assert_almost_equal(tform.translation, translation)
 
+    # test special case for scale where the rotation isn't exactly 90deg,
+    # but very close
+    scale = 1.0
+    rotation = np.pi / 2
+    translation = (0, 0)
+    params = np.array([[0, -1, 1.33226763e-15],
+                       [1, 2.22044605e-16, -1.33226763e-15],
+                       [0, 0, 1]])
+    tform = SimilarityTransform(params)
+    assert_almost_equal(tform.scale, scale)
+    assert_almost_equal(tform.rotation, rotation)
+    assert_almost_equal(tform.translation, translation)
+
 
 def test_affine_estimation():
     # exact solution
@@ -162,7 +180,7 @@ def test_affine_estimation():
 
     # via estimate method
     tform3 = AffineTransform()
-    tform3.estimate(SRC, DST)
+    assert tform3.estimate(SRC, DST)
     assert_almost_equal(tform3.params, tform2.params)
 
 
@@ -186,10 +204,13 @@ def test_affine_init():
     assert_almost_equal(tform2.shear, shear)
     assert_almost_equal(tform2.translation, translation)
 
+    # scalar vs. tuple scale arguments
+    assert_almost_equal(AffineTransform(scale=0.5).scale, AffineTransform(scale=(0.5, 0.5)).scale)
+
 
 def test_piecewise_affine():
     tform = PiecewiseAffineTransform()
-    tform.estimate(SRC, DST)
+    assert tform.estimate(SRC, DST)
     # make sure each single affine transform is exactly estimated
     assert_almost_equal(tform(SRC), DST)
     assert_almost_equal(tform.inverse(DST), SRC)
@@ -303,10 +324,36 @@ def test_projective_estimation():
 
     # via estimate method
     tform3 = ProjectiveTransform()
-    tform3.estimate(SRC, DST)
+    assert tform3.estimate(SRC, DST)
     assert_almost_equal(tform3.params, tform2.params)
 
 
+def test_projective_weighted_estimation():
+
+    # Exact solution with same points, and unity weights
+    tform = estimate_transform('projective', SRC[:4, :], DST[:4, :])
+    tform_w = estimate_transform('projective',
+                                 SRC[:4, :], DST[:4, :], np.ones(4))
+    assert_almost_equal(tform.params, tform_w.params)
+
+    # Over-determined solution with same points, and unity weights
+    tform = estimate_transform('projective', SRC, DST)
+    tform_w = estimate_transform('projective',
+                                 SRC, DST, np.ones(SRC.shape[0]))
+    assert_almost_equal(tform.params, tform_w.params)
+
+    # Repeating a point, but setting its weight small, should give nearly
+    # the same result.
+    point_weights = np.ones(SRC.shape[0] + 1)
+    point_weights[0] = 1.0e-15
+    tform1 = estimate_transform('projective', SRC, DST)
+    tform2 = estimate_transform('projective',
+                                SRC[np.arange(-1, SRC.shape[0]), :],
+                                DST[np.arange(-1, SRC.shape[0]), :],
+                                point_weights)
+    assert_almost_equal(tform1.params, tform2.params, decimal=3)
+
+
 def test_projective_init():
     tform = estimate_transform('projective', SRC, DST)
     # init with transformation matrix
@@ -321,10 +368,33 @@ def test_polynomial_estimation():
 
     # via estimate method
     tform2 = PolynomialTransform()
-    tform2.estimate(SRC, DST, order=10)
+    assert tform2.estimate(SRC, DST, order=10)
     assert_almost_equal(tform2.params, tform.params)
 
 
+def test_polynomial_weighted_estimation():
+    # Over-determined solution with same points, and unity weights
+    tform = estimate_transform('polynomial', SRC, DST, order=10)
+    tform_w = estimate_transform('polynomial',
+                                 SRC,
+                                 DST,
+                                 order=10,
+                                 weights=np.ones(SRC.shape[0]))
+    assert_almost_equal(tform.params, tform_w.params)
+
+    # Repeating a point, but setting its weight small, should give nearly
+    # the same result.
+    point_weights = np.ones(SRC.shape[0] + 1)
+    point_weights[0] = 1.0e-15
+    tform1 = estimate_transform('polynomial', SRC, DST, order=10)
+    tform2 = estimate_transform('polynomial',
+                                SRC[np.arange(-1, SRC.shape[0]), :],
+                                DST[np.arange(-1, SRC.shape[0]), :],
+                                order=10,
+                                weights=point_weights)
+    assert_almost_equal(tform1.params, tform2.params, decimal=4)
+
+
 def test_polynomial_init():
     tform = estimate_transform('polynomial', SRC, DST, order=10)
     # init with transformation parameters
@@ -339,7 +409,7 @@ def test_polynomial_default_order():
 
 
 def test_polynomial_inverse():
-    with testing.raises(Exception):
+    with pytest.raises(Exception):
         PolynomialTransform().inverse(0)
 
 
@@ -370,17 +440,17 @@ def test_union():
 def test_union_differing_types():
     tform1 = SimilarityTransform()
     tform2 = PolynomialTransform()
-    with testing.raises(TypeError):
+    with pytest.raises(TypeError):
         tform1.__add__(tform2)
 
 
 def test_geometric_tform():
     tform = GeometricTransform()
-    with testing.raises(NotImplementedError):
+    with pytest.raises(NotImplementedError):
         tform(0)
-    with testing.raises(NotImplementedError):
+    with pytest.raises(NotImplementedError):
         tform.inverse(0)
-    with testing.raises(NotImplementedError):
+    with pytest.raises(NotImplementedError):
         tform.__add__(0)
 
     # See gh-3926 for discussion details
@@ -404,43 +474,43 @@ def test_geometric_tform():
         assert(np.isfinite(dst).all())
 
 def test_invalid_input():
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         ProjectiveTransform(np.zeros((2, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         AffineTransform(np.zeros((2, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         SimilarityTransform(np.zeros((2, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EuclideanTransform(np.zeros((2, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         AffineTransform(matrix=np.zeros((2, 3)), scale=1)
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         SimilarityTransform(matrix=np.zeros((2, 3)), scale=1)
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EuclideanTransform(
             matrix=np.zeros((2, 3)), translation=(0, 0))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         PolynomialTransform(np.zeros((3, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         FundamentalMatrixTransform(matrix=np.zeros((3, 2)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(matrix=np.zeros((3, 2)))
 
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(rotation=np.zeros((3, 2)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(
             rotation=np.zeros((3, 3)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(
             rotation=np.eye(3))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(rotation=np.eye(3),
                                  translation=np.zeros((2,)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(rotation=np.eye(3),
                                  translation=np.zeros((2,)))
-    with testing.raises(ValueError):
+    with pytest.raises(ValueError):
         EssentialMatrixTransform(
             rotation=np.eye(3), translation=np.zeros((3,)))
 
@@ -449,15 +519,19 @@ def test_degenerate():
     src = dst = np.zeros((10, 2))
 
     tform = SimilarityTransform()
-    tform.estimate(src, dst)
+    assert not tform.estimate(src, dst)
+    assert np.all(np.isnan(tform.params))
+
+    tform = EuclideanTransform()
+    assert not tform.estimate(src, dst)
     assert np.all(np.isnan(tform.params))
 
     tform = AffineTransform()
-    tform.estimate(src, dst)
+    assert not tform.estimate(src, dst)
     assert np.all(np.isnan(tform.params))
 
     tform = ProjectiveTransform()
-    tform.estimate(src, dst)
+    assert not tform.estimate(src, dst)
     assert np.all(np.isnan(tform.params))
 
     # See gh-3926 for discussion details
@@ -473,6 +547,44 @@ def test_degenerate():
         # a transform could be returned with nan values.
         assert(not tform.estimate(src, dst) or np.isfinite(tform.params).all())
 
+    src = np.array([[0, 2, 0], [0, 2, 0], [0, 4, 0]])
+    dst = np.array([[0, 1, 0], [0, 1, 0], [0, 3, 0]])
+    tform = AffineTransform()
+    assert not tform.estimate(src, dst)
+    # Prior to gh-6207, the above would set the parameters as the identity.
+    assert np.all(np.isnan(tform.params))
+
+    # The tesselation on the following points produces one degenerate affine
+    # warp within PiecewiseAffineTransform.
+    src = np.asarray([
+        [0, 192, 256], [0, 256, 256], [5, 0, 192], [5, 64, 0], [5, 64, 64],
+        [5, 64, 256], [5, 192, 192], [5, 256, 256], [0, 192, 256],
+    ])
+
+    dst = np.asarray([
+        [0, 142, 206], [0, 206, 206], [5, -50, 142], [5, 14, 0], [5, 14, 64],
+        [5, 14, 206], [5, 142, 142], [5, 206, 206], [0, 142, 206],
+    ])
+    tform = PiecewiseAffineTransform()
+    assert not tform.estimate(src, dst)
+    assert np.all(np.isnan(tform.affines[4].params))  # degenerate affine
+    for idx, affine in enumerate(tform.affines):
+        if idx != 4:
+            assert not np.all(np.isnan(affine.params))
+    for affine in tform.inverse_affines:
+        assert not np.all(np.isnan(affine.params))
+
+
+def test_normalize_degenerate_points():
+    """Return nan matrix *of appropriate size* when point is repeated."""
+    pts = np.array([[73.42834308, 94.2977623 ],] * 3)
+    mat, pts_tf = _center_and_normalize_points(pts)
+    assert np.all(np.isnan(mat))
+    assert np.all(np.isnan(pts_tf))
+    assert mat.shape == (3, 3)
+    assert pts_tf.shape == pts.shape
+
+
 
 def test_projective_repr():
     tform = ProjectiveTransform()
@@ -505,3 +617,119 @@ def test_projective_str():
     want = want.replace('1\\.', ' *1\\.')
     print(want)
     assert re.match(want, str(tform))
+
+
+def _assert_least_squares(tf, src, dst):
+    baseline = np.sum((tf(src) - dst) ** 2)
+    for i in range(tf.params.size):
+        for update in [0.001, -0.001]:
+            params = np.copy(tf.params)
+            params.flat[i] += update
+            new_tf = tf.__class__(matrix=params)
+            new_ssq = np.sum((new_tf(src) - dst) ** 2)
+            assert new_ssq > baseline
+
+
+def test_estimate_affine_3d():
+    ndim = 3
+    src = np.random.random((25, ndim)) * 2 ** np.arange(7, 7 + ndim)
+    matrix = np.array(
+        [[4.8, 0.1, 0.2, 25],
+         [0.0, 1.0, 0.1, 30],
+         [0.0, 0.0, 1.0, -2],
+         [0.0, 0.0, 0.0, 1.]]
+    )
+    tf = AffineTransform(matrix=matrix)
+    dst = tf(src)
+    dst_noisy = dst + np.random.random((25, ndim))
+    tf2 = AffineTransform(dimensionality=ndim)
+    assert tf2.estimate(src, dst_noisy)
+    # we check rot/scale/etc more tightly than translation because translation
+    # estimation is on the 1 pixel scale
+    assert_almost_equal(tf2.params[:, :-1], matrix[:, :-1], decimal=2)
+    assert_almost_equal(tf2.params[:, -1], matrix[:, -1], decimal=0)
+    _assert_least_squares(tf2, src, dst_noisy)
+
+
+def test_fundamental_3d_not_implemented():
+    with pytest.raises(NotImplementedError):
+        _ = FundamentalMatrixTransform(dimensionality=3)
+    with pytest.raises(NotImplementedError):
+        _ = FundamentalMatrixTransform(np.eye(4))
+
+
+def test_array_protocol():
+    mat = np.eye(4)
+    tf = ProjectiveTransform(mat)
+    assert_equal(np.array(tf), mat)
+    assert_equal(np.array(tf, dtype=int), mat.astype(int))
+
+
+def test_affine_transform_from_linearized_parameters():
+    mat = np.concatenate(
+        (np.random.random((3, 4)), np.eye(4)[-1:]), axis=0
+    )
+    v = mat[:-1].ravel()
+    mat_from_v = _affine_matrix_from_vector(v)
+    tf = AffineTransform(matrix=mat_from_v)
+    assert_equal(np.array(tf), mat)
+    # incorrect number of parameters
+    with pytest.raises(ValueError):
+        _ = _affine_matrix_from_vector(v[:-1])
+    with pytest.raises(ValueError):
+        _ = AffineTransform(matrix=v[:-1])
+
+
+def test_affine_params_nD_error():
+    with pytest.raises(ValueError):
+        _ = AffineTransform(scale=5, dimensionality=3)
+
+
+def test_euler_rotation():
+    v = [0, 10, 0]
+    angles = np.radians([90, 45, 45])
+    expected = [-5, -5, 7.1]
+    R = _euler_rotation_matrix(angles)
+    assert_almost_equal(R @ v, expected, decimal=1)
+
+
+def test_euclidean_param_defaults():
+    # 2D rotation is 0 when only translation is given
+    tf = EuclideanTransform(translation=(5, 5))
+    assert np.array(tf)[0, 1] == 0
+    # off diagonals are 0 when only translation is given
+    tf = EuclideanTransform(translation=(4, 5, 9), dimensionality=3)
+    assert_equal(np.array(tf)[[0, 0, 1, 1, 2, 2], [1, 2, 0, 2, 0, 1]], 0)
+    with pytest.raises(ValueError):
+        # specifying parameters for D>3 is not supported
+        _ = EuclideanTransform(translation=(5, 6, 7, 8), dimensionality=4)
+    with pytest.raises(ValueError):
+        # incorrect number of angles for given dimensionality
+        _ = EuclideanTransform(rotation=(4, 8), dimensionality=3)
+    # translation is 0 when rotation is given
+    tf = EuclideanTransform(rotation=np.pi * np.arange(3), dimensionality=3)
+    assert_equal(np.array(tf)[:-1, 3], 0)
+
+
+def test_similarity_transform_params():
+    with pytest.raises(ValueError):
+        _ = SimilarityTransform(translation=(4, 5, 6, 7), dimensionality=4)
+    tf = SimilarityTransform(scale=4, dimensionality=3)
+    assert_equal(tf([[1, 1, 1]]), [[4, 4, 4]])
+
+
+def test_euler_angle_consistency():
+    angles = np.random.random((3,)) * 2 * np.pi - np.pi
+    euclid = EuclideanTransform(rotation=angles, dimensionality=3)
+    similar = SimilarityTransform(rotation=angles, dimensionality=3)
+    assert_array_almost_equal(euclid, similar)
+
+
+def test_2D_only_implementations():
+    with pytest.raises(NotImplementedError):
+        _ = PolynomialTransform(dimensionality=3)
+    tf = AffineTransform(dimensionality=3)
+    with pytest.raises(NotImplementedError):
+        _ = tf.rotation
+    with pytest.raises(NotImplementedError):
+        _ = tf.shear
