@@ -20,6 +20,14 @@ def test_pair_crop():
     assert_equal(out.shape, (6, 2))
 
 
+def test_pair_tuple_crop():
+    arr = np.arange(45).reshape(9, 5)
+    out = crop(arr, ((1, 2),))
+    assert_array_equal(out[0], [6, 7])
+    assert_array_equal(out[-1], [31, 32])
+    assert_equal(out.shape, (6, 2))
+
+
 def test_int_crop():
     arr = np.arange(45).reshape(9, 5)
     out = crop(arr, 1)
@@ -28,6 +36,14 @@ def test_int_crop():
     assert_equal(out.shape, (7, 3))
 
 
+def test_int_tuple_crop():
+    arr = np.arange(45).reshape(9, 5)
+    out = crop(arr, (1,))
+    assert_array_equal(out[0], [6, 7, 8])
+    assert_array_equal(out[-1], [36, 37, 38])
+    assert_equal(out.shape, (7, 3))
+
+
 def test_copy_crop():
     arr = np.arange(45).reshape(9, 5)
     out0 = crop(arr, 1, copy=True)
@@ -46,3 +62,11 @@ def test_zero_crop():
     arr = np.arange(45).reshape(9, 5)
     out = crop(arr, 0)
     assert out.shape == (9, 5)
+
+
+def test_np_int_crop():
+    arr = np.arange(45).reshape(9, 5)
+    out1 = crop(arr, np.int64(1))
+    out2 = crop(arr, np.int32(1))
+    assert_array_equal(out1, out2)
+    assert out1.shape == (7, 3)
